/**
 * Check to see if an action of the appropriate type is available and return its name. 
 * This method will  make the faster actions use a slower slot if they have already been used. 
 * 
 * @param {PropertyMap} combatant Check this combatant's actions.
 * @param {String} action The action being checked
 * @return {String} The action that would have to be used to satisfy the check or null if no action is available
 */
function checkAction(combatant, action) {

  // Treat no action as a free action
  action = String(action).replace(/^\s+|\s+$/g, '').toLowerCase();
  if (action == "" || action == "no action" || action == "none") return "free";
  
  // Always allow free action
  if (action == "free") return action;
  
  // Remove the interrupt/reaction from immediate actions
  if (action == "immediate interrupt" || action == "immediate reaction")
    action = "immediate";

  // Only one immediate action a round, it can't use any other action
  var available = combatant.get("actions." + action).booleanValue();
  if (action == "immediate") {
    if (available) return "immediate";
    return null;
  } // endif
  
  // If minor is used, try move
  if (!available && action == "minor") {
    action = "move";
    available = combatant.get("actions.move").booleanValue();
  } // endif
  
  // If move is used, try standard
  if (!available && action == "move") {
    available = combatant.get("actions.standard").booleanValue();
    action = "standard";
  } // endif
  if (!available) return null;
  return action;
}

/**
 * Check to see if an action of the appropriate type is available. This method will 
 * make the faster actions use a slower slot if they have already been used. 
 * 
 * @param {PropertyMap} combatant Check this combatant's actions.
 * @param {String} action The action being checked
 * @return {Boolean} The value true if an action is available.
 */
function actionAvailable(combatant, action) {
  return checkAction(combatant, action) != null;
}

/**
 * Use an action. The method does not check to see if the action is available.
 * 
 * @param {PropertyMap} combatant Use this combatant's action.
 * @param {String} action The action being used (case insensitive): standard, move, minor, immediate. The
 * actions free, no action & none are supported bug ignored. Any other value will cause an error.
 */
function useAction(combatant, action) {
  action = action.toLowerCase();
  if (action == "free" || action == "no action" || action == "none") return;
  combatant.put("actions." + action, false);
}

/**
 * Reset all of the actions
 * 
 * @param {PropertyMap} combatant Reset this combatant's actions.
 * @return {String} Output to the user.
 */
function resetActions(combatant) {
  
  // Reset all of the powers
  var change = resetAttackReady(combatant.get("powers.powers"));
  change |= resetAttackReady(combatant.get("monsterPowers"));
  
  // Any actions set?
  if (!combatant.get("actions.standard").booleanValue() || !combatant.get("actions.move").booleanValue()
      || !combatant.get("actions.minor").booleanValue() || !combatant.get("actions.immediate").booleanValue()) {
    combatant.put("actions.standard", true);
    combatant.put("actions.move", true);
    combatant.put("actions.minor", true);
    combatant.put("actions.immediate", true);
    change = true;
  } // endif
  return change ? combatant.get("name") + "'s actions have been reset." : "";
}

/**
 * Reset the attack ready flag for a list of powers
 * 
 * @param {PropertyList} powers A list of powers to be modified
 * @return A flag that indicates that at least one power was modified when true.
 */
function resetAttackReady(powers) {
  var i = powers.iterator();
  var change = false;
  while (i.hasNext()) {
    var power = i.next();
    var attackReady = power.get("attackReady").booleanValue();
    if (attackReady) {
      change = true;
      power.put("attackReady", false);
    } // endif
  } // endwhile
  return change;
}

/**
 * Add an action point to a combatant
 * 
 * @param {PropertyMap} combatant Add the action point to this combatant.
 * @return {String} Message displayed to the user.
 */
function addActionPoint(combatant) {
  var Java = new JavaImporter(java.math);
  with (Java) {
    var ap = combatant.get("actions.actionPoints").intValue() + 1;
    combatant.put("actions.actionPoints", new BigDecimal(ap));
    return combatant.get("name") + " has reached a milestone and now has " + ap + " action point" + (ap != 1 ? "s" : "") + ".";
  } // with
}

/**
 * Use a combatant's action point.
 * 
 * @param {PropertyMap} combatant Use this combatant's action point.
 * @return {String} Message displayed to the user.
 */
function useActionPoint(combatant) {
  var Java = new JavaImporter(java.math);
  with (Java) {
    var ap = combatant.get("actions.actionPoints").intValue() - 1;
    if (ap < 0) 
      return combatant.get("name") + " has no more action points.";
    combatant.put("actions.actionPoints", new BigDecimal(ap));
    combatant.put("actions.standard", true);
    return combatant.get("name") + " has used an action point to gain a standard action and has " + ap + " action point " + (ap == 1 ? "" : "s") + " remaining.";
  } // with
}

/**
 * Reset all of the encounter level powers, actions, dying, and second wind.
 * 
 * @param {PropertyMap} combatant Short rest for this combatant.
 * @param {String} Message displayed to the user.
 */
function shortRest(combatant) {
  resetActions(combatant);
  combatant.put("hp.secondWind.available", true);  
  combatant.put("hp.dying.failedFirst", false);
  combatant.put("hp.dying.failedSecond", false);
  combatant.put("hp.dying.failedThird", false);
  resetPowers(combatant, "Encounter");
  return combatant.get("name") + " has taken a short rest.";
}

/**
 * Reset one type of power 
 * 
 * @param {PropertyMap} combatant Reset this combatant's powers.
 * @param {String} The type of power to reset: encounter or daily (case insensitive)
 */
function resetPowers(combatant, type) {
	type = type.toString().toLowerCase();
  var Java = new JavaImporter(java.util);
  with (Java) {
    var i = combatant.get("powers.powers").iterator();
    while (i.hasNext()) {
      var power = i.next();
      if (String(power.get("type")).toLowerCase() == type)
        power.put("used", false);
    } // endwhile
  } // endwith
}

/**
 * Reset all of the combatants data: Second wind, dying flags, encounter & daily powers, actions,
 * damage, healing surges, magic item uses.
 * 
 * @param {PropertyMap} combatant Reset this combatant's data.
 * @return {String} Message to be displayed to the user
 */
function extendedRest(combatant) {
  var Java = new JavaImporter(java.math);
  with (Java) {
    shortRest(combatant);
    combatant.put("actions.actionPoints", BigDecimal.ONE);
    combatant.put("hp.damage", BigDecimal.ZERO);
    combatant.put("hp.temporary", BigDecimal.ZERO);
    combatant.put("hp.healingSurges.used", BigDecimal.ZERO);
    combatant.put("hp.dying.dying", false);
    combatant.put("hp.isBloodied", false);
    combatant.put("actions.firstItemDailyUse", false);
    combatant.put("actions.secondItemDailyUse", combatant.get('level').intValue() > 10 ? false : true);
    combatant.put("actions.thirdItemDailyUse", combatant.get('level').intValue() > 20 ? false : true);
    resetPowers(combatant, "Daily");
    return combatant.get("name") + " has taken an extended rest.";
  }
}

/**
 * Check to see if there is a daily item use available
 * 
 * @param {PropertyMap} combatant Check this combatant
 * @return {Boolean} Flag indicating that the item is used. 
 */
function checkItemDailyUse(combatant) {
  var Java = new JavaImporter(java.lang);
  with (Java) {
    var used = combatant.get("actions.firstItemDailyUse").booleanValue();
    used = used && (combatant.get('level').intValue > 10 ? combatant.get("actions.secondItemDailyUse").booleanValue() : true);
    used = used && (combatant.get('level').intValue > 20 ? combatant.get("actions.thirdItemDailyUse").booleanValue() : true);
    return used ? Boolean.FALSE : Boolean.TRUE;
  } // endwith
}

/**
 * Use an item's daily use power.
 * 
 * @param {PropertyMap} combatant This combatant is using a daily power from an item.
 */
function useItemDailyUse(combatant) {
  if (!combatant.get("actions.firstItemDailyUse").booleanValue()) {
    combatant.put("actions.firstItemDailyUse", true);
    return;
  } else if (!combatant.get("actions.secondItemDailyUse").booleanValue()) {
    combatant.put("actions.secondItemDailyUse", true);
    return;
  } // endif
  combatant.put("actions.thirdItemDailyUse", true);
}