/**
 * Enter damage and apply it to the combatant. Calculate wait states and bloodied as well.
 * 
 * @param {PropertyMap} hp Hit point information for a combatant. The bloodied value is modified as needed.
 * @param {Number?} fixedDamage Fixed amount of damage. When set greater than zero the user is not asked to enter damage.
 * @return {String} The message to display the the user.
 */
function enterDamage(hp, fixedDamage) {
  var Java = new JavaImporter(net.rptools.chartool.ui.charsheet.component, java.math, java.lang);
  with (Java) {
    try {
      
      // Get the value from the user and the base values from hp
      var damage = hp.get("damage").intValue();
      var temporary = hp.get("temporary").intValue();
      if (typeof fixedDamage == 'undefined' || fixedDamage == null || fixedDamage == 0) {      
        var script = CharSheetUtils.showInputNumberDialog("Enter Damage", "Enter a damage value or script:", null, hp);
        if (script == null) return "";
        script = script.intValue();
      } else {
        script = fixedDamage;
      } // endif
 
      // Remove damage from temporary first
      var message = "";
      if (temporary > 0) {
        
        // Check to see if all of the temporary hp are removed, then add the rest to damage.
        var newTemp = temporary - script;
        if (newTemp < 0) {
          damage = damage - newTemp;
          var tempRemoved = temporary;
          temporary = 0;
          message = "Removed " + tempRemoved + " temporary hit points and added " + (-newTemp) + " hit points of damage for a total of " + script + " new hit points of damage to " + hp.get("root.name") + ".";
        } else {
          temporary = newTemp;
          message = "Removed " + script + " temporary hit points from " + hp.get("root.name") + ".";
        } // endif
      } else { 
        
        // No temporary hitpoints, add it directly to damage
        damage = damage + script;
        message = "Adding " + script + " hit points of damage to " + hp.get("root.name") + ".";
      } // endif
      hp.put("damage", new BigDecimal(damage)); 
      hp.put("temporary", new BigDecimal(temporary)); 
                    
      // Check for state change if current goes under zero this round. 
      var base = hp.get("maximum").intValue();
      var bloodied = hp.get("bloodied").intValue();
      var isBloodied = hp.get("isBloodied").booleanValue();
      var waitState = hp.get("root.itWaitState");
      if ((base - damage) <= (bloodied * -1) && !"Dead".equals(waitState)) {
        message = message + "<br>" + hp.get("root.name") + " is dead.";
        hp.put("root.itWaitState", "Dead");
        isBloodied = true;
      } else if ((base - damage) < 0 && !"Dead".equals(waitState) && !"Dying".equals(waitState)) {
        message = message + "<br>" + hp.get("root.name") + " is dying.";
        hp.put("root.itWaitState", "Dying");
        hp.put("dying.dying", true);
        isBloodied = true;
      } else if ((base - damage) <= bloodied && !isBloodied) {
        message = message + "<br>" + hp.get("root.name") + " is now bloodied.";
        isBloodied = true;
      } // endif
      hp.put("isBloodied", isBloodied); 
      return message;
    } catch (e) {
      return "Error: " + e.toString(); // Show message to the user
    } // endtry
  }
}

/**
 * Enter healing and apply it to the combatant. Calculate wait states and bloodied as well.
 * 
 * @param {PropertyMap} hp Hit point information for a combatant. The bloodied value is modified as needed.
 * @param {Number?} fixedHealing Fixed amount of healing. When set greater than zero the user is not asked to enter healing unless force dialog is true.
 * @param {Boolean?} forceDialog Force display of the dialog even if the fixed healing value is set. 
 * @return {String} The message to display the the user.
 */
function enterHealing(hp, fixedHealing, forceDialog) {
  var Java = new JavaImporter(net.rptools.chartool.ui.charsheet.component, java.math, java.lang);
  with (Java) {
    try {
      
      // Get the script and the values from the map
      if (typeof fixedHealing == 'undefined' || typeof fixedHealing == 'string' || fixedHealing == null || fixedHealing == 0 || forceDialog) {     
        if (typeof fixedHealing == 'undefined' || fixedHealing == null || fixedHealing == 0)
          fixedHealing = hp.get("healingSurges.healed.current").toString();
        var script = CharSheetUtils.showInputNumberDialog("Enter Healing", "Enter the hit points healed value or script:", fixedHealing, hp);
        if (script == null) return "";
        script = script.intValue();
      } else {
        script = fixedHealing;
      } // endif
      var damage = hp.get("damage").intValue();
      var base = hp.get("maximum").intValue();
      
      // Find out the damage actually removed, can't be over max, always add from zero, then apply the damage
      var damageRemoved = Math.min(damage, script);
      if (damage > base) damage = base;
      damage = damage - script;
      if (damage < 0) damage = 0;
      
      // Generate the output message
      var message = "";
      if (damageRemoved == 0) {
        message = "No damage removed from " + hp.get("root.name") + ".";
      } else {
        message = "Removed " + damageRemoved + " hit points of damage from " + hp.get("root.name") + ".";
      } // endif
      
      // See if the combatant has left the dying state.
      var waitState = hp.get("root.itWaitState");      
      if ((base - damage) > 0 && "Dying" == waitState || "Dead" == waitState) {
        hp.put("root.itWaitState", "");
        message = message + "<br>" + hp.get("root.name") + " is now conscious.";
        hp.put("dying.dying", false);
        hp.put("dying.failedFirst", false);
        hp.put("dying.failedSecond", false);
        hp.put("dying.failedThird", false);
      } // endif
      
      // See if the combatant has left the bloodied state.
      var bloodied = hp.get("bloodied").intValue();
      var isBloodied = hp.get("isBloodied").booleanValue();
      if ((base - damage) > bloodied && isBloodied) {
        message = message + "<br>" + hp.get("root.name") + " is no longer bloodied.";
        isBloodied = false;
      } // endif
      
      // Save the modified values and ouput the message
      hp.put("isBloodied", isBloodied); 
      hp.put("damage", new BigDecimal(damage)); 
      return message;
    } catch (e) {
      return "Error: " + e.toString(); // Show message to the user
    } // endtry
  }
}

/**
 * Use a healing surge to remove damage. Also adjust bloodied and wait states 
 * 
 * @param {PropertyMap} hp Hit point information for a combatant. The surges used and bloodied values are modified as needed.
 * @param {Number?} fixedHealing Fixed amount of healing.
 * @param {Boolean?} forceDialog Force display of the dialog even if the fixed healing value is set. 
 * @return {String} The message to display the the user.
 */
function useHealingSurge(hp, fixedHealing) {
  var Java = new JavaImporter(java.math);
  with (Java) {
    var surges = hp.get("healingSurges.base").intValue();
    var surgesUsed = hp.get("healingSurges.used").intValue();
    if ((surges - surgesUsed) <= 0)
      return "Error: " + hp.get("root.name") + " has no healing surges left for the day.";
    var message = enterHealing(hp, fixedHealing, true);
    if (message == "") return "";
    if (message.substring(0, 6) != "Error:")
      hp.put("healingSurges.used", new BigDecimal(surgesUsed + 1));
    return message;
  }
}

/**
 * Use second wind remove damage. Use a healing surge and adjust bloodied and wait states as well.
 * 
 * @param {PropertyMap} hp Hit point information for a combatant. The surges used and bloodied values are modified as needed.
 * @return {String} The message to display the the user.
 */
function useSecondWind(hp) {
  var Java = new JavaImporter(java.lang, net.rptools.chartool.model.property);
  with (Java) {
    var secondWind = hp.get("secondWind.available").booleanValue();
    if (secondWind != true)
      return "Error: " + hp.get("root.name") + " has already used his second wind for this encounter.";
    var action = checkAction(hp.get("root"), hp.get("secondWind.action"));
    if (action == null) 
      return "Error: " + hp.get("root.name") + " has already used his standard action for this round.";
    message = useHealingSurge(hp, hp.get("secondWind.amount.diceString"), true);
    if (message == "") return "";
    if (message.substring(0, 6) != "Error:") {
      message = message.substring(0, message.length - 1) + " by using Second Wind";
      hp.put("secondWind.available", false);
      useAction(hp.get("root"), hp.get("secondWind.action"));
    } // endif
    return message;
  } // endwith
}

/**
 * Enter temporary hit points. Temporary hit points do not stack.
 * 
 * @param {PropertyMap} hp Hit point information for a combatant.
 * @param {Number?} fixedHitpoints Fixed amount of temporary hit points. When greater than zero the user is not asked for input.
 * @return {String} The message to display the the user.
 */
function enterTemporary(hp, fixedHitpoints) {
  var Java = new JavaImporter(net.rptools.chartool.ui.charsheet.component, java.math, java.lang);
  with (Java) {
    try {
      var script = 0;
      
      // Get the hit points from the user or just use those passed
      if (!fixedHitpoints) {
        script = CharSheetUtils.showInputNumberDialog("Enter Temporary", "Enter the temporary hit points added as a value or script:", hp.get("healingSurges.healed.current"), hp);
        if (script == null) return "";
        script = script.intValue();
      } else {
        script = fixedHitpoints;
      } // endif
      
      // Compare to existing hit points, temporary hit points do not stack.
      var temporary = hp.get("temporary").intValue();
      if (script > temporary) {
        var message = '';
        if (temporary == 0) {
          message = hp.get("root.name") + ' now has ' + script + ' temporary hit points.';
        } else {
          message = hp.get("root.name") + ' now has ' + (script - temporary) + ' more temporary hit points. Temporary hit points do not stack.';
        } 
        hp.put("temporary", new BigDecimal(script)); 
        return message;
      } else {
        return hp.get("root.name") + ' receives no extra temporary hit points. Temporary hit points do not stack.'
      } // endif
    } catch (e) {
      return "Error: " + e.toString(); // Show message to the user
    } // endtry
  }
}  

/**
 * Make a saving throw when in the dying state.
 * 
 * @param {PropertyMap} hp Hit point information for a combatant.
 * @return The message to display to the user.
 */
function makeSavingThrow(hp) {
  var Java = new JavaImporter(net.rptools.chartool.ui.charsheet.component, java.math, java.lang, net.rptools.common.expression);
  with (Java) {
    var roll = Math.floor(Math.random() * 20) + 1;
    var message = '1d20 &#8658; ' + roll + '';
    if (roll >= 10)
      return hp.get("root.name") + " has saved against dying this round. " + message;
    var flagNames = new Array("dying.failedFirst", "dying.failedSecond", "dying.failedThird");
    var set = false;
    var count = 0;
    for(var index = 0; index < flagNames.length; index++) {
      var flag = hp.get(flagNames[index]).booleanValue();
      if (!set && !flag) {
        flag = true;
        hp.put(flagNames[index], true);
        set = true;
      } // endif
      if (flag) count = count + 1;
    } // endfor
    if (count < 3) {
      return hp.get("root.name") + " has now failed " + count + " save" + (count == 1 ? "" : "s") + " against dying. " + message;
    } else {
      hp.put("root.itWaitState", "Dead");
      return hp.get("root.name") + " has failed the third save against dying and is now dead. " + message;
    } // endif
  }  // endwith
}