/**
 * Adjust a value by removing the abilty and half level modifiers
 * 
 * @param {String} property The name of the property to be set.
 * @param {PropertyMap} map The map that is being modified.
 * @param {String} value The value to be modified. Must convert to a number.
 * @param {String} ability The ability score used to calculate a modifer on the value. Must convert to a number.
 * @param {String} level The level of the monster for the half level modifier. Must convert to a number.
 * @param ?{BigDecimal} mod An optional modifier added to the value.
 */
function adjustAndSetValue(property, map, value, ability, level, mod) {
  var Java = new JavaImporter(net.rptools.chartool.model.db, java.math);
  with (Java) {

    // Make sure everything is set
    if (!value) value = 0;
    if (!ability) ability = 0;
    if (!level) level = 0;
    
    // Do the adjusting
    var adjusted = value * 1;
    adjusted -= Math.floor(((ability * 1) - 10) / 2);
    adjusted -= Math.floor((level * 1) / 2);
    if (mod) adjusted += mod;
    map.put(property, new BigDecimal(String(adjusted)));
  } // endwith
}

/**
 * Load the resistances, vulnerabilities, and immunities of a monster. Each one goes
 * in its own 'resistance' slot instance.
 * 
 * @param {PropertyMap} map The map that is being modified.
 */
function loadResistances(map) {
  var Java = new JavaImporter(net.rptools.chartool.model.property, net.rptools.chartool.model.db, java.math);
  with (Java) {
    var res = map.get('resistances.manual');
    
    // Each Immunity is separated by a comma w/ no amount
    value = String(map.get('monsterImmunities'));
    if (value) {
      var immunities = value.split(',');
      for(var i = 0; i < immunities.length; i++) {
        var keyword = findKeyword(immunities[i]);
        if (keyword != null) {
          var spv = new SlotPropertyValue("resistances");
          spv.put('type', 'Immunity');
          spv.put('keyword', keyword);
          res.add(spv);
        } // endif
      } // endfor
    } // endif
      
      // The save value is just a number but applies to all types of save
    value = String(map.get('monsterSaves'));
    if (value && value != '0') {
      var spv = new SlotPropertyValue("resistances");
      spv.put('type', 'Saving Throw');
      spv.put('keyword', 'Any');
      spv.put('amount', new BigDecimal(value));
      res.add(spv);
    } // endif
    
    // Resistances and Vulnerabilities are separated by commas and have a keyword and amount.
    loadResistanceAndAmount(map, map.get('monsterResistances'), 'Resistance');
    loadResistanceAndAmount(map, map.get('monsterVulnerabilities'), 'Vulnerability');
  } // endwith
}

/**
 * Load the resistances & vulnerabilities which have keyword/amount pairs.
 * 
 * @param {PropertyMap} map The map that is being modified.
 * @param {String} value The value being loaded.
 * @param {String} type The type of data being saved.
 */
function loadResistanceAndAmount(map, value, type) {
  var Java = new JavaImporter(net.rptools.chartool.model.property, net.rptools.chartool.model.db, java.math);
  with (Java) {
    if (!value) return; // Nothing to do 
    
    // Split into the individual resistaince/vulnerability
    var rvs = value.split(',');
    for(var i = 0; i < rvs.length; i++) {
      var rv = rvs[i].split(' ');
      var amount = parseInt(rv[0]);
      var keyword = findKeyword(rv[0]);
       
      // Make sure it is in the expected format, or punt
      if (rv.length == 2 && amount != NaN && amount > 0) {
        var spv = new SlotPropertyValue("resistances");
        spv.put('type', type);
        spv.put('keyword', keyword);
        spv.put('amount', new BigDecimal(String(amount)));
        res.add(spv);
      } // endif
    } // endfor
  } // endwith    
}

/**
 * Find a supported keyword in the passed value.
 * 
 * @param {String} keyword Look for a supported keyword in this string.
 * @return {String} The keyword found in the proper case & w/o extra spaces. 
 */
function findKeyword(type) {
  if (!type) return;
  type = String(type);
  var t = type.replace(/^\s+|\s+$/g, '');
  for(var i = 0; i < availableDamageTypes.length; i++) {
    if (availableDamageTypes[i].toLowerCase() == t) return availableDamageTypes[i]; 
  } // endfor
  return null;
}

/**
 * All of the skills and the name of the ability that they use.
 */
var availableSkills = {
  'acrobatics' :  'dex', 
  'arcana' :  'int', 
  'athletics' :  'str', 
  'bluff' :  'cha', 
  'diplomacy' :  'cha',
  'endurance' :  'con', 
  'heal' :  'wis', 
  'history' :  'int', 
  'insight' :  'wis', 
  'intimidate' :  'cha',
  'nature' :  'wis', 
  'perception' :  'wis', 
  'religion' :  'int', 
  'stealth' :  'dex', 
  'streetwise' :  'cha',
  'theivery' :  'dex'
};

/**
 * Load all of the skills in the property into the character.
 * 
 * @param {PropertyMap} map The map that is being modified.
 */
function loadSkills(map) {
  
  // Get the value, needs to exist
  rpValue = String(map.get('monsterSkills')).toLowerCase();
  if (!rpValue) return; // Nothing to do
  
  // Parse the skills one at a time.
  var pattern = /(\w+)\s*([+-]\d+)/g;
  while ((skill = pattern.exec(rpValue)) != null) {
    skill[1] = skill[1].replace(/^\s+|\s+$|\+/g, '');
    var sDef = availableSkills[skill[1]];
    if (sDef) {
      skill[2] = skill[2].replace(/^\s+|\s+$|\+/g, '');
      adjustAndSetValue(skill[1] + '.userMod', map, skill[2], map.get(sDef + '.base').intValue(), map.get('level').intValue());
    } // endif
    if (typeof s == 'string') continue; // No skill found
  } // endwhile
}

/**
 * The valid ability names
 */
var availableAbilities = ['str', 'con', 'dex', 'int', 'wis', 'cha'];

/**
 * Parse the abilities from the property and set their base values.
 * 
 * @param {PropertyMap} map The map that is being modified.
 */
function parseAbilities(map) {
  var Java = new JavaImporter(net.rptools.chartool.model.db, java.math);
  with (Java) {
    rpValue = String(map.get('monsterAbilities'));
    map.remove('monsterAbilities');
    rpValue = rpValue.toLowerCase();
    var pattern = /(\w+)\s*(\d+)\s*\([+-]\d*\)/g;
    while ((ability = pattern.exec(rpValue)) != null) {
      map.put(ability[1] + '.base', new BigDecimal(ability[2]));
    } // endwhile
  } // endwith
}

/**
 * Load a combatant from the data in the combatant lookup table.
 * 
 * @param {PropertyMap} map The map for the new combatant.
 */
function loadCombatant(map) {
  
  // Get the abilities
  parseAbilities(map);
  var level = map.get('level').intValue();
  var wis = map.get('wis.base').intValue();
  var dex = map.get('dex.base').intValue();
  var intel = map.get('int.base').intValue();
  var str = map.get('str.base').intValue();
  var con = map.get('con.base').intValue();
  var cha = map.get('cha.base').intValue();
  
  // Get the perception, init modifier and defenses
  adjustAndSetValue("perception.base", map, map.get('monsterPerception'), wis, level);
  map.remove('monsterPerception');
  adjustAndSetValue("itInitiativeModifier", map, map.get('initMod'), dex, level);
  map.remove('initMod');
  adjustAndSetValue("armorClass.userMod", map, map.get('monsterAC'), Math.max(dex, intel), level);
  map.remove('monsterAC');
  adjustAndSetValue("fortitude.userMod", map, map.get('monsterFortitude'), Math.max(str, con), level);
  map.remove('monsterFortitude');
  adjustAndSetValue("reflex.userMod", map, map.get('monsterReflex'), Math.max(dex, intel), level);
  map.remove('monsterReflex');
  adjustAndSetValue("reflex.userMod", map, map.get('monsterReflex'), Math.max(dex, intel), level);
  map.remove('monsterReflex');
  adjustAndSetValue("will.userMod", map, map.get('monsterWill'), Math.max(wis, cha), level);
  map.remove('monsterWill');
  
  // Immunities, resistances, vulnerabilities, saves & Skills
  loadResistances(map);
  loadSkills(map);
  
  // Load all of the powers
  var Java = new JavaImporter(net.rptools.chartool.model.db, net.rptools.lib, net.rptools.chartool.model.property, 
                              net.rptools.chartool.ui.component); 
  with (Java) {
    var table = PropertyTable.getPropertyTable(PropertySettings.getInstance().getDatabaseName(), 'monsterPower');
    var condition = new SimpleCondition('MONSTER', SimpleCondition.Operator.EQUALS, map.get('name'));
    var list = table.selectRows(0, 0, null, condition);    
    if (!list.isEmpty()) {
      var monsterPowers = map.get('monsterPowers');
      for(var i =0 ; i < list.size(); i++) {
        var spv = new SlotPropertyValue(list.get(i), 'monsterPower');
        spv.setName(list.get(i).get('name'));
        monsterPowers.add(spv);
      } // endfor
    } // endif
    
    // Load the icons and portrait
    var icon = map.get("itToken");
    if (icon != null && !(icon instanceof RPIcon)) {
      icon = RPIconFactory.getInstance().get(String(icon));
      map.nodePut("itToken", icon, false);
    } // endif
    icon = map.get("rpPortrait");
    if (icon != null && !(icon instanceof RPIcon)) {
      icon = RPIconFactory.getInstance().get(String(icon));
      map.nodePut("rpPortrait", icon, false);
    }
  } // endwith
}

/**
 * This function generates the start of the HTML that is loaded into the monster power fields.
 */
function startHtml() {
  var Java = new JavaImporter(net.rptools.chartool.model.property);
  with (Java) {
    var html = '<html><head><base href="';
    html += PropertySettings.getInstance().getClassLoader().getURLs()[0];
    html += '"/></head><body style="font-family: sans-serif; font-size: 12pt">';
    return html;
  }
}

/**
 * This function generates the start of the HTML that is loaded into the monster power fields.
 */
function endHtml() {
  return ('</body></html>')
}

/**
 * Get the title HTML text of a monster power.
 * 
 * @param {CharSheetController} controller The controller that is displaying the power.
 */
function setPowerTitle(controller) {
  if (!controller) return 'No Power Data';
  power = controller.getData();
  var display = String(power.get('display')); 
  var action = String(power.get('action')); 
  var trigger = String(power.get('trigger'));
  var sustain = String(power.get('sustain'));
  var type = String(power.get('type'));
  var rechargeWhen = String(power.get('rechargeWhen'));
  var keywords = String(power.get('keywords'));
  
  // Base text starts with the power name highlighted.
  var html = startHtml() + setPowerIcon(power) + '<b>' + display + '</b> ';
  
  // Determing if there is any action text
  var text = action ? action : '';  
  text += trigger ? (text ? '; ' : '') + trigger : '';  
  text += sustain ? (text ? '; ' : '') + 'sustain ' + sustain : '';  
  text += type ? (text ? '; ' : '') + type : '';
  
  // Add the text or icons if recharging
  if (type.toLowerCase() == 'recharge') {
    if (parseInt(rechargeWhen) != NaN) {
      rechargeWhen = parseInt(rechargeWhen);
      text += ' <span style="font-family:D&D 4e icons">'
      for(var i = rechargeWhen; i < 7; i++)
        text += i + ' ';
      text += '</span>' 
    } else {
      text += 's ' + rechargeWhen;
    } // endif
  } // endif
  
  // Add the parnthesis aroung the text if needed and add it to the html
  if (text) html += '(' + text + ')';
  
  // Output keywords if needed, then finish the html
  if (keywords) html += '<b> &#9830; ' + keywords + '</b>';
  return html + endHtml();
}

/**
 * Get the title HTML text of a monster power.
 * 
 * @param {PropertyMap} power Get the title for this power,
 */
function setPowerText(controller) {
  if (!controller) return 'No Power Data';
  power = controller.getData();
  var attackTypeAndRange = String(power.get('attackTypeAndRange')); 
  var attack = String(power.get('attack'));
  var defense = String(power.get('defense'));
  var hit = String(power.get('hit'));
  var miss = String(power.get('miss'));
  var secondaryAttack = String(power.get('secondaryAttack'));
  var secondaryDefense = String(power.get('secondaryDefense'));
  var secondaryHit = String(power.get('secondaryHit'));
  var effect = String(power.get('effect'));
  
  // Check for attack info first, only add if more than one word
  var text = '';
  if (attackTypeAndRange.split(/\s+/g).length != 1)
    text = attackTypeAndRange; 
  
  // The attack, defense & hit
  if (attack) {
    text += (text ? '; ' : '') + attack;
    if (defense) text += ' vs. ' + defense;
    text += hit ? (text ? '; ' : '') + hit : '';
  }
  
  // The secondary attack, defense & hit
  if (secondaryAttack) {
    text += (text ? '; ' : '') + secondaryAttack + ' vs. ' + secondaryDefense;
    text += secondaryHit ? (text ? '; ' : '') + secondaryHit : '';
  }
  
  // Miss and effect then done
  text += miss ? (text ? '; ' : '') + 'Miss: ' + miss : '';
  text += effect ? (text ? '; ' : '') + effect : '';
  return startHtml() + text + endHtml();
}

/**
 * Determine what icon is displayed for a power
 * 
 * @param {JComponent} power Get the icon for this power as a styled string.
 */
function setPowerIcon(power) {
  if (!power) return 'No Power Data';
  var attackTypeAndRange = String(power.get('attackTypeAndRange')).toLowerCase(); 
  var basicAttack = power.get('basicAttack').booleanValue();
    
  // Determine icon name from attack type
  var icon = ''; 
  if (attackTypeAndRange.indexOf('melee') >= 0) {
    icon = 'm';
  } else if (attackTypeAndRange.indexOf('ranged') >= 0) {
    icon = 'r';
  } else if (attackTypeAndRange.indexOf('close') >= 0) {
    icon = 'c';
  } else if (attackTypeAndRange.indexOf('area') >= 0) {
    icon = 'a';
  } // endif
    
  // Is this a basic attack?
  if (icon) {
    if (basicAttack) {
      icon = icon.toUpperCase();
    } // endif
  } else {
     return ''; // No icon
  } // endif
  return '<span style="font-family:D&D 4e icons;font-size:16px">' + icon + '</span>&nbsp;&nbsp;'
}

/**
 * Use a monster power. Evaluates first attack if the power is an attack power.
 * 
 * @param {CharSheetController} controller The controller that is displaying the power.
 * @param {Boolean} secondary Using the secondary power?
 */
function useMonsterPower(controller, secondary) {
  if (!controller) return 'No Power Data';
  var power = controller.getData();
  var combatant = power.get('root');  
  var msg = combatant.get("name") + " is using the " + power.get('display') + (secondary ? " secondary" : "") + ' power.';
  
  // Attack power?
  var attack = String(power.get(secondary ? 'secondaryAttack' : 'attack'));
  if (attack) {
    msg += '<br/><i>' + (secondary ? "Secondary " : "") + 'Attack:</i> ' + attack + ' vs. ' + power.get(secondary ? 'secondaryDefense' : 'defense');
    var roll = power.get(secondary ? 'secondaryAttackModifier' : 'attackModifier').intValue();
    if (roll == 0) {
      roll = '1d20'; 
    } else {
      roll = '1d20 + ' + roll;
    } // endif
    msg += '; ' + evaluateRoll(roll)[0];
    
    // Add hit text
    var hit = String(power.get(secondary ? 'secondaryHit' : 'hit'));
    if (hit) {
      msg += '<br/><i>' + (secondary ? "Secondary " : "") + 'Hit:</i> ' + hit;
      
      // Calculate damage 
      var hitDamage = String(power.get(secondary ? 'secondaryHitDamage' : 'hitDamage'));
      if (hitDamage.indexOf('d') < 0) {
        if (hitDamage != '0' && hitDamage != '')
          msg += '<br/><i>' + (secondary ? "Secondary " : "") + 'Damage:</i> ' + hitDamage;
      } else {
        msg += '<br/><i>' + (secondary ? "Secondary " : "") + 'Damage:</i> ' + evaluateRoll(hitDamage)[0];
        var critDamage = [hitDamage, 0, 0, ''];
        evaluateMaxDieRoll(critDamage);
        msg += '<br/><i>' + (secondary ? "Secondary " : "") + 'Crit:</i> ' + critDamage[2];
      } // endif

      // Add damage types if available
      var keywords = String(power.get('keywords'));
      keywords = keywords.split(/[\s,]+/);
      msg += findDamageTypes(keywords);
    } // endif    
    
    // Add miss 
    var miss = String(power.get('miss'));
    if (miss)
      msg += '<br/><i>Miss:</i> ' + miss;
  } // endif

  // Add effect  
  var effect = String(power.get('effect'));
  if (effect)
    msg += '<br/><i>Effect:</i> ' + effect;
      
  // Use the power & action, make the attacks ready
  if (!secondary) {
    power.put("attackReady", true);
    if (power.get("type").toLowerCase() != "at-will")
      power.put("used", true);
    var action = checkAction(combatant, power.get("action"));
    if (action == null) {
      msg += "<br/>There isn't a " + power.get("action") + " action available.";
    } else {
      useAction(combatant, action);
    } // endif
  } // endif
  controller.setFields();
  return msg;
}

/**
 * Use a monster attack. Used for attacks that hit multiple targets.
 * 
 * @param {CharSheetController} controller The controller that is displaying the power.
 * @param {Boolean} secondary Using the secondary power?
 */
function useMonsterAttack(controller, secondary) {
  if (!controller) return 'No Power Data';
  var power = controller.getData();
  var attack = String(power.get(secondary ? 'secondaryAttack' : 'attack'));
  var msg = power.get('display') + (secondary ? " Secondary" : "") + ' Attack: '; 
  if (attack) {
    msg += attack + ' vs. ' + power.get(secondary ? 'secondaryDefense' : 'defense');
    var roll = power.get(secondary ? 'secondaryAttackModifier' : 'attackModifier').intValue();
    if (roll == 0) {
      roll = '1d20'; 
    } else {
      roll = '1d20 + ' + roll;
    } // endif
    msg += '; ' + evaluateRoll(roll)[0];
  } else {
    msg += ' not an attack power.';
  } // endif
  return msg;
}

/**
 * Evaluate an attack or damage roll for a monster power. 
 * 
 * @param {String} roll The die roll to be evaluated.
 * @return {Array} An array containing the display string first and the actual roll second.
 */  
function evaluateRoll(roll) {
  var Java = new JavaImporter(net.rptools.common.expression);
  with (Java) {
    
    // If no die roll set the fixed value info
    var ep = new ExpressionParser();
    var r = ep.evaluate(roll);
    var msg = '' + roll + ' &#8658; ' + r.getDetailExpression();
    if (String(r.getDetailExpression()) != String(r.getValue()))
      msg += " = " + r.getValue().toString();
    msg += '';
    return [msg, r.getValue().intValue()];
  }
}

/**
 * Look for damage types in the passed keywords.
 * 
 * @param {Array[String]} keywords The array of keywords to be searched.
 * @return {String} The damage type determined from the keywords.
 */
function findDamageTypes(keywords) {
  
  // Search the passed keywords for damage types.
  var dmg = [];
  for (i in keywords) {
    var k = keywords[i].toLowerCase();
    for (j in availableDamageTypes) {
      if (k == availableDamageTypes[j].toLowerCase()) {
        dmg.push(availableDamageTypes[j]);
        break;
      } // endif
    } // endfor
  } // endfor
  
  // Convert the result into a string
  if (dmg.length == 0) return '';
  if (dmg.length == 1) return ' ' + dmg[0];
  msg = ' ';
  for (damage in dmg) {
    if (damage == dmg[dmg.length - 1]) msg += ' and '
    else if (damage != dmg[0]) msg += ', ';
    msg += damage;
  } // endfor
  return msg;
}

/**
 * Recharge a power that needs it.
 *  
 * @param {CharSheetController|PropertyMap} power The power being recharged or the controller that is displaying the power.
 */
function rechargePower(power) {
  if (!power) return 'No Power Data';
  if (power instanceof net.rptools.chartool.ui.charsheet.CharSheetController) {
    var controller = power;   
    power = controller.getData();
  } // endif
  if (power.get('type').toLowerCase() != 'recharge' || !power.get("used"))
    return 'No recharge needed.';
  var rechargeWhen = power.get('rechargeWhen');
  var msg = 'Recharging ' + power.get('display') + ': ';
  if (rechargeWhen instanceof java.math.BigDecimal) {
    var roll = evaluateRoll('1d6');
    msg += roll[0] + ' ';
    if (roll[1] >= rechargeWhen.intValue()) {
      power.put("used", false);
      msg += 'Success!';
    } else {
      msg += 'Fail';
    } // endif
  } else if (rechargeWhen != 'never') {
    power.put("used", false);
    msg += rechargeWhen;
  } // endif
  if (controller) controller.setFields();
  return msg;
}

/**
 * Recharge all of a monster's powers that need it.
 * 
 * @param {PropertyMap} combatant The combatant that needs its powers recharged.
 */
function rechargeAll(combatant) {
  var powers = combatant.get('monsterPowers');
  var i = powers.iterator();
  var m = '';
  while (i.hasNext()) {
    var power = i.next();
    if (power.get('type') == 'recharge' && power.get('used').booleanValue()) 
      m = m + (!m ? '' : '<br/>') + rechargePower(power);
  } // endwhile
  return m;
}