/**
 * All of the avaialble damage type keywords
 */
var availableDamageTypes = new Array("", "Acid", "Cold", "Fire", "Force", "Lightning", "Necrotic", "Poison", "Psychic", "Radiant", "Thunder");

/**
 * Die size constants.
 */
var INCREASE_DIE_SIZE = 'increaseDieSize';
var ONE_HAND_DIE_SIZE = new Array('1d4', '1d6', '1d8', '1d10', '1d12', '2d6', '2d8', '2d10');
var TWO_HAND_DIE_SIZE = new Array('1d8', '2d4', '1d10', '1d12', '2d6', '2d8', '2d10');

/**
 * Make a single attack with a power. This method does not check for actions so that multiple attacks can be made.
 *
 * @param {PropertyMap} combatant Combatant making the power attack.
 * @param {PropertyMap} power The power being used.
 * @param {String} damageSet The damage set being done: primary, secondary or tertiary
 * @return {String} Text message displayed to the user.
 */
function powerAttack(combatant, power, damageSet) {
  damageSet = damageSet.toLowerCase();
  var roll = Math.floor(Math.random() * 20) + 1;
  var mods = power.get((damageSet == "primary" ? "t" : damageSet + "T") + "otalAttackModifier").intValue();
  var msg = combatant.get("name") + "'s " + power.get('name');
  if (damageSet != "primary") msg += " " + damageSet;
  return msg + ' attack roll is 1d20 + ' + mods + ' &#8658; ' + roll + ' + ' + mods + ' = ' + (roll + mods) + '';
}

/**
 * Use a power.
 *
 * @param {PropertyMap} combatant Combatant using the power.
 * @param {PropertyMap} power The power being used.
 * @param {String} damageSet The damage set being done: primary, secondary or tertiary
 * @return {String} Text message displayed to the user.
 */
function usePower(combatant, power, damageSet) {
  damageSet = damageSet.toLowerCase();
  var Java = new JavaImporter(java.lang, net.rptools.chartool.model.property);
  with (Java) {
  
    // Power used?
    var message = "";
    if (power.get("used").booleanValue() && damageSet == "primary")
      return combatant.get("name") + " has already used the " + power.get("name") + " power for the "
             + (String(power.get("type")).toLowerCase() == "encounter" ? "encounter." : "day.")
    if (!power.get("available").booleanValue() && damageSet == "primary")
      return combatant.get("name") + " can not use the " + power.get("name") + " right now because they don't meet a requirement or have an action.";

    // Utility power?
    if (("" + power.get("attack")).replace(/^\s+|\s+$/g, '') == "") {
      message = combatant.get("name") + " is using the " + power.get('kind') + " power " + power.get("name") + "\n";
    } else {
    
      // Attack power, roll to hit, roll for regular damage & critical damage
      message = powerAttack(combatant, power, damageSet);
      message += "<br/>" + powerDamage(combatant, power, damageSet, false, false);
      message += "<br/>" + powerDamage(combatant, power, damageSet, true, false);
    } // endif

    // Use the power and action
    var powerType = String(power.get("type")).toLowerCase();
    if (powerType != "at-will") power.put("used", true);
    if (damageSet == "primary") {
      var action = checkAction(combatant, power.get("action"));
      useAction(combatant, action);
      if (String(power.get('kind')).toLowerCase() == "item" && powerType == "daily")
        useItemDailyUse(combatant);
    } // endif
    
    // Call the use script
    var temp = power.executeScript("use");
    if (temp != null && temp.length() > 0) {
      if (message.length > 0) message = message + "<br/>";
      message = message + temp
    } // endif
    if (damageSet == "primary")
      power.put("attackReady", true);
    return message;
  } // endwith
}

/**
 * Find damage types in power keywords
 *
 * @param {String} keywords Search for damage types in these keywords
 * @return {Array} Array of strings containing the damage type names.
 */
function findDamageType(keywords) {
  keywords = keywords.toLowerCase();
  var type = [];
  for(var i = 1; i < availableDamageTypes.length; i++) {
    if (keywords.indexOf(availableDamageTypes[i].toLowerCase()) >= 0)
      type.push(availableDamageTypes[i]);
  } // endfor
  return type;
}

/**
 * Calcualte the damage for a specfic damage set of a power for both display and to make the roll.
 *
 * @param {PropertyMap} combatant The combatant using the power.
 * @param {PropertyMap} power Calculate the damage for this power.
 * @param {String} damageSet The set of damage to be calculated: primary, secondary, or tertiary
 * @param {Boolean} critical Flag indicating that critical damage is being calculated.
 * @param {Boolean} display Flag indicating that damage is being calculated for display and not rolled.
 * @return {String} Message displayed to the user.
 */
function powerDamage(combatant, power, damageSet, critical, display) {
  damageSet = damageSet.toLowerCase();
  var Java = new JavaImporter(java.util, java.math);
  with (Java) {
    
    // If not an attack return ""
    if (String(power.get((damageSet == "primary" ? "a" : damageSet + "A") + "ttack")).replace(/^\s+|\s+$/g, '') == "") return "";

    // Primary doesn't have a prefix.
    var dmgSet = damageSet + "H";
    if (damageSet == "primary")
      dmgSet = "h";
    var weapon = (combatant.get("currentWeapon.useOffHandWeapon").booleanValue() ? "offHand" : "weapon");
    var implementPower = String(power.get("keywords")).toLowerCase().indexOf("implement") >= 0; 
    var holySymbol = combatant.containsKey("equipped.holySymbol.name") && String(combatant.get("class.implements")).toLowerCase().indexOf('holy symbol') >= 0;
    var implementWeaponGroup = String(combatant.get("class.implements")).toLowerCase().indexOf(String(combatant.get(weapon + ".group")).toLowerCase()) >= 0;
    var implementWeaponName = String(combatant.get("class.implements")).toLowerCase().indexOf(String(combatant.get(weapon + ".name")).toLowerCase()) >= 0;
    var validImplement = implementPower && (implementWeaponGroup || implementWeaponName || holySymbol);
    var weaponAttack = power.get(dmgSet + "itWeaponDamageMultiplier").intValue() != 0;
    var b = 0;
    
    // Calculate the normal damage first, it is always needed
    var nwdm = weaponDamageMods(combatant.get(weapon + ".normalDamageBonus.modifiers"));
    if (!weaponAttack && validImplement && holySymbol)
      nwdm = getHolySymbolModifier(combatant.get("equipped.holySymbol"), 'normalDamageBonus');
    var nd = nwdm.get("");
    if (nd == null) {
      nd = new Array("", 0, 0, "");
      nwdm.put("", nd);
    } // endif
    if (combatant.get(weapon + ".normalDamageBonus.userMod") != null)
      nd[1] = nd[1] + combatant.get(weapon + ".normalDamageBonus.userMod").intValue();
    if (weaponAttack) {

      // Check for damage size changes then get the damage die
      var ids = nwdm.get(INCREASE_DIE_SIZE);
      var count = (ids == null ? 0 : ids[1]) + adjustWeaponForSize(String(combatant.get("size")));
      var dd = resizeWeapon(combatant.get(weapon), count);
      var d = power.get(dmgSet + "itWeaponDamageMultiplier").intValue() * dd[0];
      var brutal = brutalWeapon(combatant.get('equipped.' + weapon))
      nd[0] = d + "d" + dd[1] + (brutal > 0 ? 'r' + brutal : '') + (nd[0].length > 0 ? " + " + nd[0] : "");
      
      // Extra damage die bonus for weapons
      var hd = String(power.get(dmgSet + "itDamage"));
      if (hd) nd[0] = nd[0] + (nd[0].length > 0 ? " + " : "") + hd;
    } else {
      
      // Check for a valid implement
      if (!validImplement) {
        nwdm.clear(); // Not valid, so no mods
        nd = new Array("", 0, 0, "");
        nwdm.put("", nd);
      } // endif
      nd[0] = nd[0] + (nd[0].length > 0 ? " + " : "") + power.get(dmgSet + "itDamage");
    } // endif
    nd[1] += power.get(dmgSet + "itDamageModifier").intValue();
    nd[1] += applyGeneralPowerMods(combatant, power, dmgSet + "itDamageModifier");
    var dmg = nwdm;
    
    // Need critical damage?
    if (critical && (validImplement || weaponAttack)) {
      var cd = null;
      var cwdm = weaponDamageMods(combatant.get(weapon + ".criticalDamageBonus.modifiers"));
      if (!weaponAttack && validImplement && holySymbol)
        cwdm = getHolySymbolModifier(combatant.get("equipped.holySymbol"), 'criticalDamageBonus');
      
      // Combine the max normal damage w/ the existing crit damage
      for (var i = 0; i < availableDamageTypes.length; i++) {
        nd = nwdm.get(availableDamageTypes[i]);
        cd = cwdm.get(availableDamageTypes[i]);
        if (nd == null && cd == null) continue;
        
        // Make sure there is normal damage available, fake it if there isn't
        if (nd != null) {
          evaluateMaxDieRoll(nd);
        } else {
          nd = new Array("", 0, 0, "");
        } // endif
        if (cd == null) {
          cwdm.put(availableDamageTypes[i], new Array("", nd[2], 0, ""));
        } else {
          cd[1] = cd[1] + nd[2];
        } // endif
      } // endfor

      // Check for high crit
      if (weaponAttack && combatant.get(weapon + ".highCrit").booleanValue()) {
        var hc = combatant.get(weapon + ".damageDieCount").intValue() * (Math.floor((combatant.get("level").intValue() - 1) / 10) + 1)
        hc = hc + "d" + combatant.get(weapon + ".damageDieSize").intValue()
        cd = cwdm.get("");
        if (cd == null) {
          cwdm.put("", new Array(hc, 0, 0, ""));
        } else {
          cd[0] = cd[0].length == 0 ? hc : cd[0] + " + " + hc;
        } // endif
      } // endif
      dmg = cwdm;
    } else if (critical) {
      evaluateMaxDieRoll(nd);
      nd = new Array("", nd[2], 0, "");
      nwdm.put("", nd);
    } // endif

    // Check the power to see if it has a damage type. Untyped damage is copied to the proper types
    var s = "";
    b = 0;
    var count = 0;
    var damageTypes = findDamageType(power.get("keywords"));
    if (damageTypes.length == 1) {
      
      // Only one type, add the untyped to that type
      d = dmg.remove("");
      if (d != null) {
        td = dmg.get(damageTypes[0]);
        if (td == null) {
          dmg.put(damageTypes[0], d);
        } else {
          td[0] = td[0].length == 0 ? d[0] : td[0] + (d[0].length > 0 ? " + " + d[0] : "");
          td[1] = td[1] + d[0];
        } // endif
      } // endif
    } // endif
    
    // Create the string
    for(var i = 0; i < availableDamageTypes.length; i++) {
      d = dmg.get(availableDamageTypes[i]);
      if (d == null) continue;
      count = count + 1;
      if (display) {
        if (s.length > 0) s = s + " + ";
        s = s + d[0];
        if (d[1] != 0) {
          if (s.length > 0) {
            if (d[1] > 0) {
              s = s + " + ";
            } else {
              s = s + " - ";
              d[1] = Math.abs(d[1]);
            } // endif
          } // endif
          s = s + d[1];
        } // endif
      } else {
        evaluateDieRoll(d);
        b = b + d[2];
        if (s.length > 0) s = s + " + ";
        s = s + d[3];
      } // endif
      if (availableDamageTypes[i].length > 0) {
        s = s + " (" + availableDamageTypes[i] + ")";
      } else if (damageTypes.length > 0) {
        if (display) {
          s = s + " (" + damageTypes.join("/") + ")";
        } else {
          
          // Split the multiple damage types
          var dSplit = Math.floor(d[0] / damageTypes.length);
          var dFirst = dSplit + d[0] % damageTypes.length;
          s = s + " (" + dFirst + " " + damageTypes[0];
          for (var j = 1; j < damageTypes.length; j++)
            s = s + ", " + dSplit + damageTypes[j];
          s = s + ")";
        } // endif
      } else if (dmg.size() > 1) {
        s = s + " (Untyped)"
      } // endif
    } // endfor
    var msg = s;
    if (!display) {
      msg = combatant.get("name") + "'s ";
      if (damageSet != "primary")
        msg += damageSet + " ";
      if (critical)
        msg += "critical ";
      msg += " damage roll is: ";
      if (count > 1)
        msg += "<u>" + b + " total damage:</u>  ";
      msg += s;
    } // endif
    return msg;
  } // endwith
}

/**
 * Add all damage modifiers of the same type together.
 *
 * @param {PropertyList} modifiers The modifiers being combined
 * @return {HashMap} Map of the combined modifiers. The key is the damage type and the value is an
 * array containing the die rolls in the first element and the constant modifier in the second. The third and
 * fourth elements are created but not used by this method.
 */
function weaponDamageMods(modifiers) {
  var Java = new JavaImporter(java.util, java.math, java.lang);
  with (Java) {
    
    // Place the data for each type into this map. The values are are arrays of die strings & numeric modifiers
    var td = new HashMap();
    
    // If there are any mods, load at each of them, ignore the ones that don't apply
    if (modifiers != null && !modifiers.isEmpty()) { // any mods?

      // Prune damage that doesn't stack
      var i = modifiers.iterator();
      var appliedMod = new HashMap();
      var list = new Array();
      while (i.hasNext()) {
        var mod = i.next();
        if (!mod.get("applies").booleanValue()) continue;
        
        // Always apply untyped mods, penalties, stackable damage and dice damage
        var modType = mod.get("type");
        var modSource = mod.get("source");
        if (modType == null || modType.length() == 0 || modSource == INCREASE_DIE_SIZE || mod.get('stack').booleanValue()
            || !(mod.get("bonus") instanceof BigDecimal) || mod.get("bonus").intValue() < 0) {
          list.push(mod);
          continue;
        } // endif
        
        // Find the better one and save it
        var c = appliedMod.get(modType) == null ? 0 : appliedMod.get(modType).get('bonus').intValue();
        var b = mod.get("bonus").intValue();
        if (b > c) appliedMod.put(modType, mod);
      } // endwhile
      
      // Add the best back to the list
      i = appliedMod.values().iterator();
      while (i.hasNext())
        list.push(i.next())
      
      // Now read mods that still apply and add them to the proper damage types in source
      for(i=0; i<list.length; i++) {
        mod = list[i];
        modSource = mod.get("source");
        
        // Get the damage type array from the map, create it if it doesn't exist.
        var t = td.get(modSource)
        if (t == null) {
          t = new Array("", 0, 0, ""); // die roll, fixed, total, display
          td.put(modSource, t);
        } // endif
        
        // Get the bonus, handle by type
        var b = mod.get("bonus");
        if (b instanceof BigDecimal) {
          t[1] = t[1] + b.intValue();
          continue;
        } else if (typeof b == "string" || b instanceof String) {
          if (t[0].length != 0) t[0] = t[0] + " + ";
          t[0] = t[0] + b;
        } else {
          print("Unexpected error: unknown type = " + typeof b);
        } // endif'
      } // endfor
    } // endif
    return td;
  } // endwith
}

/**
 * Evaluate all of the dice rolls in the passed data by turning them into multiplies instead
 * Add the constant value as well.
 *
 * @param {Array} The data array containing the die roll in the first element and the constant in the second.
 * @return {Array} The data passed in with the third element modified to contain the maximum die roll.
 */
function evaluateMaxDieRoll(data) {
  
  // If no die roll set the fixed value info
  var s = data[0];
  if (s.length == 0) {
    data[2] = data[1];
    return data;
  } // endif

  // Replace the die rolls w/ multiplies after adding fixed then evaluate
  if (data[1] != 0)
    s = s + " + " + data[1];
  s = s.replace(/(\d+)[dD](\d+)/g, "$1 * $2") + ";";
  data[2] = eval(s);
  return data;
}

/**
 * Evaluate all of the dice rolls in the passed data.
 *
 * @param {Array} data The first element in this array is a die roll, which is passed to
 * the script parser and then added to the value in the second element. The result is
 * placed into the third element and the detailed string is placed into the fourth.
 */
function evaluateDieRoll(data) {
  var Java = new JavaImporter(net.rptools.common.expression);
  with (Java) {
    
    // If no die roll set the fixed value info
    var s = data[0];
    if (s.length == 0) {
      data[2] = data[1];
      data[3] = "" + data[1];
      return;
    } // endif

    // Need to make a die roll, add the fixed mods to the details first
    if (data[1] != 0)
      s = s + " + " + data[1];
    var ep = new ExpressionParser();
    var r = ep.evaluate(s);
    data[2] = r.getValue().intValue();
    data[3] = '' + s + ' &#8658; ' + r.getDetailExpression();
    if (String(r.getDetailExpression()) != String(r.getValue()))
      data[3] = data[3] + " = " + data[2];
    data[3] = data[3] + '';
  }
}

/**
 * Calculate a weapon's damage die by checking for increase die size modifiers.
 *
 * @param {PropertyMap} weapon Calculate the damage die roll for this weapon
 * @param {PropertyList} modifiers A list of damage modifiers applied to the weapon. Only
 * INCREASE_DIE_SIZE modifiers that apply are used.
 * @param {String} size The size of the combatant
 * @return {String} The die roll to be made for the weapon.
 */
function getWeaponDamageDie(weapon, modifiers, size) {
  if (modifiers == null || modifiers.isEmpty())
    return weapon.get('damageDieCount') + 'd' + weapon.get('damageDieSize');
  var j = modifiers.iterator();
  var count = size ? adjustWeaponForSize(size) : 0;
  while (j.hasNext()) {
    var mod = j.next();
    if (!mod.get("applies").booleanValue() || mod.get('source') != INCREASE_DIE_SIZE) continue;
    count += mod.get('bonus').intValue();
  } // endwhile
  if (count == 0)
    return weapon.get('damageDieCount') + 'd' + weapon.get('damageDieSize');
  return resizeWeapon(weapon, count, true);
}

/**
 * Resize a weapon for large creatures of for class features that require that.
 *
 * @param {PropertyMap} weapon Increase the die size of this weapon.
 * @param {Number} count The number of times to increase the size.
 * @param {Boolean} display Should the result be for display?
 * @param {String|Array} A string with the die roll if display is set, or an Array containing the
 * die count and die size in the first two elements.
 */
function resizeWeapon(weapon, count, display) {
  if (!(weapon instanceof net.rptools.chartool.model.property.PropertyMap))
    return display ? "" : [0, 0];
  var damageDieCount = weapon.get('damageDieCount');
  var damageDieSize = weapon.get('damageDieSize');
  if (count != 0) {
    var damageDie = damageDieCount + "d" + damageDieSize;
    var damageDieSizeArray = weapon.get("hands").intValue() == 1 ? ONE_HAND_DIE_SIZE : TWO_HAND_DIE_SIZE;
    for(var i = 0; i < damageDieSizeArray.length; i++) {
      if (damageDie == damageDieSizeArray[i]) {
        var j = i + count;
        if (j < 0) j = 0;
        if (j >= damageDieSizeArray.length) j = damageDieSizeArray.length - 1;
        damageDieCount = Number(damageDieSizeArray[j].charAt(0));
        damageDieSize = Number(damageDieSizeArray[j].substring(2));
        break;
      } // endif
    } // endfor
  } // endif
  if (display) return damageDieCount + "d" + damageDieSize;
  return new Array(damageDieCount, damageDieSize);
}

/**
 * Calculate the total attack modifier for a power's damage set
 *
 * @param {PropertyMap} combatant Calculate the modifier for this combatant.
 * @param {SlotPropertyValue} power Calculate the value for this modifier.
 * @param {String} damageSet The name of the damage modifier being calculated: primary, secondary, or tertiary
 * @return {String|BigDecimal} The total attack modifier if the power is an attack, otherwise an empty string.
 */
function calculateTotalAttackModifier(combatant, power, damageSet) {
  damageSet = damageSet.toLowerCase();
  var attack = String(power.get((damageSet == "primary" ? "a" : damageSet + "A") + "ttack"));
  if (attack == '') return '';
  var halfLevel = Math.floor(combatant.get("level").intValue() / 2);
  var weapon = (combatant.get("currentWeapon.useOffHandWeapon").booleanValue() ? "offHand" : "weapon");
  var implementPower = String(power.get("keywords")).toLowerCase().indexOf("implement") >= 0; 
  var holySymbol = implementPower && combatant.containsKey("equipped.holySymbol.name") && String(combatant.get("class.implements")).toLowerCase().indexOf('holy symbol') >= 0;
  var implementWeaponGroup = String(combatant.get("class.implements")).toLowerCase().indexOf(String(combatant.get(weapon + ".group")).toLowerCase()) >= 0;
  var implementWeaponName = String(combatant.get("class.implements")).toLowerCase().indexOf(String(combatant.get(weapon + ".name")).toLowerCase()) >= 0;
  var validImplement = implementPower && (implementWeaponGroup || implementWeaponName);
  var weaponAttack = String(power.get("keywords")).toLowerCase().indexOf("weapon") >= 0;  
  var attackBonus = validImplement || weaponAttack ? combatant.get(weapon + ".attackBonus.current").intValue() : 0;
  attackBonus -= validImplement ? combatant.get(weapon + ".proficiency").intValue() : 0; // No weapon proficency for implements
  attackBonus = holySymbol ? getHolySymbolModifier(combatant.get("equipped.holySymbol"), 'attackBonus') : attackBonus;
  attackBonus += (validImplement || holySymbol) ? applyGeneralPowerMods(combatant, power, (damageSet == "primary" ? "i" : damageSet + "I") + "mplementAttackBonus") : 0;
  var attackModifier = power.get((damageSet == "primary" ? "a" : damageSet + "A") + "ttackModifier").intValue();
  var armorPenalty = combatant.get("armor.proficiencyPenalty.penalty").intValue();
  var generalPowerMod = applyGeneralPowerMods(combatant, power, (damageSet == "primary" ? "t" : damageSet + "T") + "otalAttackModifier");
  var mod = halfLevel + attackBonus + attackModifier + armorPenalty + generalPowerMod;
  return new java.math.BigDecimal(mod);
}

/**
 * Apply a general power modification to a specific power's property.
 *
 * @param {PropertyMap} combatant Apply the general power modifiers for this combatant.
 * @param {SlotPropertyValue} power Calculate the general power modifier for this power.
 * @param {String} appliedTo Calculate the general power modifier for this property in the power
 * @param {Number} The modifier to be applied for the passed power.
 */
function applyGeneralPowerMods(combatant, power, appliedTo) {
  var Java = new JavaImporter(net.rptools.chartool.model.property, java.util, java.math);
  with (Java) {
    var tot = 0;
    var modifiers = combatant.get("powers.generalPowerMods.modifiers");
    if (modifiers != null && !modifiers.isEmpty()) {
      var global = PropertySettings.getInstance().getGlobals();
      global.put("power", power);
      var appliedMod = new HashMap();
      var i = modifiers.iterator();
      while (i.hasNext()) {
        var mod = i.next();
        if (!mod.get("applies").booleanValue() || mod.get("appliedToChild") != appliedTo) continue;
        var modType = mod.get("type");
        if (modType == null || modType.length() == 0)
          modType = mod.get("source");
        if (modType == null || modType.length() == 0)
          modType = "Untyped";
        var c = appliedMod.get(modType) == null ? 0 : appliedMod.get(modType).intValue();
        var b = mod.get("bonus").intValue();
        if (mod.get("stack") || b < 0) { // penalties always stack
          b = c + b;
          modType = "stacked-" + modType;
        } else {
          b = b > c ? b : c;
        } // endif
        appliedMod.put(modType, b);
      } // endwhile
      i = appliedMod.values().iterator();
      while (i.hasNext()) {
        tot = tot + i.next().intValue();
      } // endwhile
      global.remove("power");
    } // endif
    return tot;
  } // endwith
}

/**
 * The available creature sizes used to resize weapons.
 */
var avaialbleCreatureSizes = ['Tiny', 'Small', 'Medium', 'Large', 'Huge', 'Gargantuan'];

/**
 * Calculate the number of times to resize a weapon for larger creatures
 *
 * @param {String} size The size of the combatant.
 * @return {Number} The number of times to increase the die size.
 */
function adjustWeaponForSize(size) {
  size = size.toLowerCase();
  for(var i = 0; i < avaialbleCreatureSizes.length; i++) {
    if (size == avaialbleCreatureSizes[i].toLowerCase()) {
      return i < 2 ? 0 : i - 2;
    } // endif
  } // endif
  return 0;
}

/**
 * Get a modifier for a holy symbol.
 *
 * @param {SlotPropertyValue} holySymbol The spv containing the holy symbol which is handled as a weapon.
 * @param {String} modifierType The type of modifier needed: normalDamageBonus, critcalDamageBonus, attackBonus
 * @return {HashMap|Number} For the damage values a hashmap containing the proper damage in and array and a number for
 * the attack bonus.
 */
function getHolySymbolModifier(holySymbol, modifierType) {
	
	// Get a modifier instance for the holy symbol
  var modifierSetName = String(holySymbol.get("modifierSetName"));
  var appliedTo = 'weapon.' + modifierType;
  var hints = new java.util.HashMap();
  hints.put(net.rptools.chartool.model.property.Utils.INSTANCE_VALUE_NUMBER_PROP, new java.math.BigDecimal(holySymbol.get('level').intValue()));
  var mod = net.rptools.chartool.model.property.Utils.getModifier(modifierSetName, appliedTo, null, hints);
  bonus = mod.get('bonus');
  
  // Handle bonus by requested type, damage needs to be a map with an array containing the damage.
  if (modifierType.indexOf('Damage') >= 0) {
    var source = mod.get('source');
    if (bonus instanceof java.lang.String) {
      mod = [String(bonus), 0, 0, ""];
    } else {
      mod = ["", bonus.intValue(), 0, ""];
    } // endif
    hints.clear();
    hints.put(source, mod);
    return hints;
  } else {
  	
  	// The attack value is just an integer.
    return bonus.intValue();
  } // endif
}